#import "HelloVesViewController.h"


#include <vesMapper.h>
#include <vesActor.h>
#include <vesBlend.h>
#include <vesDepth.h>

#import "mvesView.h"
#import "mvesShaderPrograms.h"
#import "mvesPolyDataMapper.h"
#import "mvesRenderer.h"

#import "HelloVesSceneFactory.h"
#import "HelloVesSceneTableViewController.h"


@interface HelloVesViewController() <HelloVesSceneTableViewControllerDelegate>

@property (nonatomic, weak) IBOutlet mvesView *mmvesView;

@property NSDate *timestamp;
@property (nonatomic, strong) UIPopoverController *pc;
@property (nonatomic, strong) id<HelloVesScene> scene;
@property (nonatomic, weak) IBOutlet UILabel *fpsLabel;

@end


@implementation HelloVesViewController

- (void)viewDidLoad
{
    [super viewDidLoad];    
    self.mmvesView.renderer->setBackgroundColor(0.1, 0.0, 0.2);
    [self.mmvesView addDefaultGestureRecognizers];
    [self setSceneName:HelloVesSceneFactory.sharedInstance.scenes[0]];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return YES;
}

- (void)update 
{
    [self.scene update];
    [self.fpsLabel setText:[NSString stringWithFormat:@"Framerate is %d for %d cells", (int)(1 / -[self.timestamp timeIntervalSinceNow]), self.scene.cellCount]];
    self.timestamp = [NSDate date];
}

- (void)setSceneName:(NSString *)name
{
    // remove old actors
    [self.scene removeActorsFromRenderer:self.mmvesView.renderer];
    
    // create scene and add actors
    self.scene = [HelloVesSceneFactory.sharedInstance createScene:name];
    [self.scene addActorsToRenderer:self.mmvesView.renderer];
    
    // reset view
    self.mmvesView.renderer->resetCamera();
    self.mmvesView.renderer->resetCameraClippingRange();
}

- (void)dismissPopoverForScene:(NSString *)name
{
    [self setSceneName:name];
    [self.pc dismissPopoverAnimated:YES];
    self.pc = nil;
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if (self.pc != nil) {
        [self.pc dismissPopoverAnimated:YES];
    }
    
    if([@"Scene" isEqualToString:segue.identifier]) {
        self.pc = [(UIStoryboardPopoverSegue *)segue popoverController];
        ((HelloVesSceneTableViewController *)segue.destinationViewController).presentingVC = self;
    }
}

@end
